<?php

namespace Drupal\funder_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'funder_field' field type.
 *
 * @FieldType(
 *   id = "funder_field",
 *   module = "funder_field",
 *   label = @Translation("Funder field"),
 *   category = @Translation("General"),
 *   description = @Translation("Funder field type with permission-based access control."),
 *   default_widget = "funder_field_widget_default",
 *   default_formatter = "funder_field_default"
 * )
 */

class FunderFieldItem extends \Drupal\Core\Field\FieldItemBase
{

    /**
     * @inheritDoc
     */
    public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition)
    {
      $properties['funder_name'] = DataDefinition::create('string')
        ->setLabel(t('Funder name'));
      $properties['funder_id'] = DataDefinition::create('string')
        ->setLabel(t('Funder ID'));
      $properties['funder_doi'] = DataDefinition::create('string')
        ->setLabel(t('Funder DOI'));
      $properties['award'] = DataDefinition::create('string')
        ->setLabel(t('Award'));
      $properties['award_url'] = DataDefinition::create('string')
        ->setLabel(t('Award URL'));
      $properties['funder_url'] = DataDefinition::create('string')
        ->setLabel(t('Funder URL'));
      return $properties;
    }

    /**
     * @inheritDoc
     */
    public static function schema(FieldStorageDefinitionInterface $field_definition)
    {
      $columns = [
        'funder_name' => [
          'type' => 'varchar',
          'length' => 255,
        ],
        'funder_id' => [
          'type' => 'varchar',
          'length' => 255,
        ],
        'funder_doi' => [
          'type' => 'varchar',
          'length' => 2048,
        ],
        'award' => [
          'type' => 'varchar',
          'length' => 2048,
        ],
        'award_url' => [
          'type' => 'varchar',
          'length' => 2048,
        ],
        'funder_url' => [
          'type' => 'varchar',
          'length' => 2048,
        ],
      ];
      return [
        'columns' => $columns,
        // @DCG Add indexes here if necessary.
      ];
    }

  /**
   * {@inheritdoc}
   */
    public function isEmpty() {
      if ($this->funder_name !== NULL && $this->funder_name !== '') {
        return FALSE;
      }
      return TRUE;
    }
}
